@@ -2,32 +2,33 @@
 :mod:`sqlparams` is a utility package for converting between various SQL
 parameter styles.
 """
+from __future__ import annotations
 
 import re
 from typing import (
 	Any,
-	AnyStr,
-	Dict,
-	Iterable,
-	List,
-	Optional,
-	Pattern,
-	Sequence,
-	Tuple,
-	Type,
-	Union)
+	Dict,  # Replaced by `dict` in 3.9.
+	Iterable,  # Replaced by `collections.abc.Iterable` in 3.9.
+	List,  # Replaced by `list` in 3.9.
+	Optional,  # Replaced by `X | None` in 3.10.
+	Pattern,  # Replaced by `re.Pattern` in 3.9.
+	Sequence,  # Replaced by `collections.abc.Sequence` in 3.9.
+	Tuple,  # Replaced by `tuple` in 3.9.
+	Type,  # Replaced by `type` in 3.9.
+	Union)  # Replaced by `X | Y` in 3.10.
 
 from . import _converting
 from . import _styles
 from . import _util
+from .typing import (
+	TSqlStr)
 
 from ._meta import (
 	__author__,
 	__copyright__,
 	__credits__,
 	__license__,
-	__version__,
-)
+	__version__)
 
 _BYTES_ENCODING = 'latin1'
 """
@@ -39,9 +40,9 @@ DEFAULT_COMMENTS: Sequence[Union[str, Tuple[str, str]]] = (
 	"--",
 )
 """
-The default comment styles to strip. This strips single line comments
-beginning with :data:`"--"` and multiline comments beginning with
-:data:`"/*"` and ending with :data:`"*/"`.
+The default comment styles to strip. This strips single line comments beginning
+with ``"--"`` and multiline comments beginning with ``"/*"`` and ending with
+``"*/"``.
 """
 
 
@@ -69,47 +70,45 @@ class SQLParams(object):
 		"""
 		Instantiates the :class:`.SQLParams` instance.
 
-		*in_style* (:class:`str`) is the parameter style that will be used
-		in an SQL query before being parsed and converted to :attr:`.SQLParams.out_style`.
+		*in_style* (:class:`str`) is the parameter style that will be used in an SQL
+		query before being parsed and converted to :attr:`.SQLParams.out_style`.
 
-		*out_style* (:class:`str`) is the parameter style that the SQL query
-		will be converted to.
+		*out_style* (:class:`str`) is the parameter style that the SQL query will be
+		converted to.
 
-		*escape_char* (:class:`str`, :class:`bool`, or :data:`None`) is the
-		escape character used to prevent matching an in-style parameter. If
-		:data:`True`, use the default escape character (repeat the initial
-		character to escape it; e.g., "%%"). If :data:`False`, do not use an
-		escape character. Default is :data:`None` for :data:`False`.
+		*escape_char* (:class:`str`, :class:`bool`, or ``None``) is the escape
+		character used to prevent matching an in-style parameter. If ``True``, use
+		the default escape character (repeat the initial character to escape it;
+		e.g., "%%"). If ``False``, do not use an escape character. Default is
+		``None`` for ``False``.
 
-		*expand_tuples* (:class:`bool` or :data:`None`) is whether to
-		expand tuples into a sequence of parameters. Default is :data:`None`
-		to let it be determined by *out_style* (to maintain backward
-		compatibility). If *out_style* is a numeric or ordinal style, expand
-		tuples by default (:data:`True`). If *out_style* is a named style,
-		do not expand tuples by default (:data:`False`).
+		*expand_tuples* (:class:`bool` or ``None``) is whether to expand tuples into
+		a sequence of parameters. Default is ``None`` to let it be determined by
+		*out_style* (to maintain backward compatibility). If *out_style* is a
+		numeric or ordinal style, expand tuples by default (``True``). If
+		*out_style* is a named style, do not expand tuples by default (``False``).
 
-		.. NOTE:: Empty tuples will be safely expanded to :data:`(NULL)` to
-		   prevent SQL syntax errors,
+		.. NOTE:: Empty tuples will be safely expanded to ``(NULL)`` to prevent SQL
+		   syntax errors,
 
-		*strip_comments* (:class:`Sequence`, :class:`bool`, or :data:`None`)
-		whether to strip out comments and what style of comments to remove.
-		If a :class:`Sequence`, this defines the comment styles. A single
-		line comment is defined using a :class:`str` (e.g., :data:`"--"` or
-		:data:`"#"`). A multiline comment is defined using a :class:`tuple`
-		of :class:`str` (e.g., :data:`("/*", "*/")`). In order for a comment
-		to be matched, it must be the first string of non-whitespace
-		characters on the line. Trailing comments are not supported and will
-		be ignored. A multiline comment will consume characters until the
-		ending string is matched. If :data:`True`, :data:`DEFAULT_COMMENTS`
-		will be used (:data:`"--"` and :data:`("/*", "*/")` styles). Default
-		is :data:`None` to not remove comments.
+		*strip_comments* (:class:`~collections.abc.Sequence`, :class:`bool`, or
+		``None``) whether to strip out comments and what style of comments to
+		remove. If a :class:`~collections.abc.Sequence`, this defines the comment
+		styles. A single line comment is defined using a :class:`str` (e.g., ``"--"``
+		or ``"#"``). A multiline comment is defined using a :class:`tuple` of
+		:class:`str` (e.g., ``("/*", "*/")``). In order for a comment to be matched,
+		it must be the first string of non-whitespace characters on the line.
+		Trailing comments are not supported and will be ignored. A multiline comment
+		will consume characters until the ending string is matched. If ``True``,
+		``DEFAULT_COMMENTS`` will be used (``"--"`` and ``("/*", "*/")`` styles).
+		Default is ``None`` to not remove comments.
 
 		The following parameter styles are supported by both *in_style* and
 		*out_style*:
 
 		-	For all named styles the parameter keys must be valid `Python identifiers`_.
-			They cannot start with a digit. This is to help prevent
-			incorrectly matching common strings such as datetimes.
+			They cannot start with a digit. This is to help preven incorrectly
+			matching common strings such as date-times.
 
 			Named styles:
 
@@ -117,28 +116,27 @@ class SQLParams(object):
 
 					... WHERE name = :name
 
-			-	"named_dollar" indicates parameters will use the named dollar
-				sign style::
+			-	"named_dollar" indicates parameters will use the named dollar sign
+				style::
 
 					... WHERE name = $name
 
 				.. NOTE:: This is not defined by `PEP 249`_.
 
-			-	"pyformat" indicates parameters will use the named Python
-				extended format style::
+			-	"pyformat" indicates parameters will use the named Python extended
+				format style::
 
 					... WHERE name = %(name)s
 
-				.. NOTE:: Strictly speaking, `PEP 249`_ only specifies
-				   "%(name)s" for the "pyformat" parameter style so only that
-				   form (without any other conversions or flags) is supported.
+				.. NOTE:: Strictly speaking, `PEP 249`_ only specifies "%(name)s" for
+				   the "pyformat" parameter style so only that form (without any other
+				   conversions or flags) is supported.
 
-		-	All numeric styles start at :data:`1`. When using a
-			:class:`~collections.abc.Sequence` for the parameters, the 1st
-			parameter (e.g., ":1") will correspond to the 1st element of the
-			sequence (i.e., index :data:`0`). When using a :class:`~collections.abc.Mapping`
-			for the parameters, the 1st parameter (e.g., ":1") will correspond
-			to the matching key (i.e., :data:`1` or :data:`"1"`).
+		-	All numeric styles start at ``1``. When using a :class:`~collections.abc.Sequence`
+			for the parameters, the 1st parameter (e.g., ":1") will correspond to the
+			1st element of the sequence (i.e., index ``0``). When using a
+			:class:`~collections.abc.Mapping` for the parameters, the 1st parameter
+			(e.g., ":1") will correspond to the matching key (i.e., ``1`` or ``"1"``).
 
 			Numeric styles:
 
@@ -146,8 +144,8 @@ class SQLParams(object):
 
 					... WHERE name = :1
 
-			-	"numeric_dollar" indicates parameters will use the numeric
-				dollar sign style (starts at :data:`1`)::
+			-	"numeric_dollar" indicates parameters will use the numeric dollar sign
+				style (starts at ``1``)::
 
 					... WHERE name = $1
 
@@ -155,17 +153,15 @@ class SQLParams(object):
 
 		- Ordinal styles:
 
-			-	"format" indicates parameters will use the ordinal Python format
-				style::
+			-	"format" indicates parameters will use the ordinal Python format style::
 
 					... WHERE name = %s
 
-				.. NOTE:: Strictly speaking, `PEP 249`_ only specifies "%s" for
-				   the "format" parameter styles so only that form (without any
-				   other conversions or flags) is supported.
+				.. NOTE:: Strictly speaking, `PEP 249`_ only specifies "%s" for the
+				   "format" parameter styles so only that form (without any other
+				   conversions or flags) is supported.
 
-			-	"qmark" indicates parameters will use the ordinal question mark
-				style::
+			-	"qmark" indicates parameters will use the ordinal question mark style::
 
 					... WHERE name = ?
 
@@ -175,10 +171,10 @@ class SQLParams(object):
 		"""
 
 		if not isinstance(in_style, str):
-			raise TypeError("in_style:{!r} is not a string.".format(in_style))
+			raise TypeError(f"{in_style=!r} is not a string.")
 
 		if not isinstance(out_style, str):
-			raise TypeError("out_style:{!r} is not a string.".format(out_style))
+			raise TypeError(f"{out_style=!r} is not a string.")
 
 		in_obj = _styles.STYLES[in_style]
 		out_obj = _styles.STYLES[out_style]
@@ -190,7 +186,7 @@ class SQLParams(object):
 		elif isinstance(escape_char, str):
 			use_char = escape_char
 		else:
-			raise TypeError("escape_char:{!r} is not a string or bool.")
+			raise TypeError(f"{escape_char=!r} is not a string or bool.")
 
 		if expand_tuples is None:
 			expand_tuples = not isinstance(out_obj, _styles.NamedStyle)
@@ -218,70 +214,67 @@ class SQLParams(object):
 			out_style=out_style,
 		)
 		"""
-		*__converter* (:class:`._converting.Converter`) is the parameter
-		converter to use.
+		*__converter* (:class:`._converting.Converter`) is the parameter converter
+		to use.
 		"""
 
 		self.__escape_char: Optional[str] = use_char
 		"""
-		*__escape_char* (:class:`str` or :data:`None`) is the escape
-		character used to prevent matching an in-style parameter.
+		*__escape_char* (:class:`str` or ``None``) is the escape character used to
+		prevent matching an in-style parameter.
 		"""
 
 		self.__expand_tuples: bool = expand_tuples
 		"""
-		*__expand_tuples* (:class:`bool`) is whether to convert tuples into
-		a sequence of parameters.
+		*__expand_tuples* (:class:`bool`) is whether to convert tuples into a
+		sequence of parameters.
 		"""
 
 		self.__in_obj: _styles.Style = in_obj
 		"""
-		*__in_obj* (:class:`._styles.Style`) is the in-style parameter
-		object.
+		*__in_obj* (:class:`._styles.Style`) is the in-style parameter object.
 		"""
 
 		self.__in_regex: Pattern = in_regex
 		"""
-		*__in_regex* (:class:`re.Pattern`) is the regular expression used to
-		extract the in-style parameters.
+		*__in_regex* (:class:`re.Pattern`) is the regular expression used to extract
+		the in-style parameters.
 		"""
 
 		self.__in_style: str = in_style
 		"""
-		*__in_style* (:class:`str`) is the parameter style that will be used
-		in an SQL query before being parsed and converted to :attr:`.SQLParams.out_style`.
+		*__in_style* (:class:`str`) is the parameter style that will be used in an
+		SQL query before being parsed and converted to :attr:`.SQLParams.out_style`.
 		"""
 
 		self.__out_obj: _styles.Style = out_obj
 		"""
-		*__out_obj* (:class:`._styles.Style`) is the out-style parameter
-		object.
+		*__out_obj* (:class:`._styles.Style`) is the out-style parameter object.
 		"""
 
 		self.__out_style: str = out_style
 		"""
-		*__out_style* (:class:`str`) is the parameter style that the SQL
-		query will be converted to.
+		*__out_style* (:class:`str`) is the parameter style that the SQL query will
+		be converted to.
 		"""
 
 		self.__strip_comment_regexes: List[Pattern] = self.__create_strip_comment_regexes(
 			strip_comments=strip_comments,
 		)
 		"""
-		*__strip_comment_regexes* (:class:`list` of :class:`Pattern`)
-		contains the regular expressions to strip out comments.
+		*__strip_comment_regexes* (:class:`list` of :class:`Pattern`) contains the
+		regular expressions to strip out comments.
 		"""
 
 		self.__strip_comments: Optional[Sequence[Union[str, Tuple[str, str]]]] = strip_comments
 		"""
-		*__strip_comments* (:class:`Sequence` or :data:`None`) contains the
-		comment styles to remove.
+		*__strip_comments* (:class:`~collections.abc.Sequence` or ``None``) contains
+		the comment styles to remove.
 		"""
 
 	def __repr__(self) -> str:
 		"""
-		Returns the canonical string representation (:class:`str`) of this
-		instance.
+		Returns the canonical string representation (:class:`str`) of this instance.
 		"""
 		return "{}.{}({!r}, {!r})".format(
 			self.__class__.__module__,
@@ -303,21 +296,20 @@ class SQLParams(object):
 		"""
 		Create the parameter style converter.
 
-		*escape_char* (:class:`str` or :data:`None`) is the escape character
-		used to prevent matching an in-style parameter.
+		*escape_char* (:class:`str` or ``None``) is the escape character used to
+		prevent matching an in-style parameter.
 
-		*expand_tuples* (:class:`bool`) is whether to convert tuples into a
-		sequence of parameters.
+		*expand_tuples* (:class:`bool`) is whether to convert tuples into a sequence
+		of parameters.
 
 		*in_obj* (:class:`._styles.Style`) is the in-style parameter object.
 
 		*in_style* (:class:`str`) is the in-style name.
 
-		*in_regex* (:class:`re.Pattern`) is the regular expression used to
-		extract the in-style parameters.
+		*in_regex* (:class:`re.Pattern`) is the regular expression used to extract
+		the in-style parameters.
 
-		*out_obj* (:class:`._styles.Style`) is the out-style parameter
-		object.
+		*out_obj* (:class:`._styles.Style`) is the out-style parameter object.
 
 		*out_style* (:class:`str`) is the out-style name.
 
@@ -333,10 +325,7 @@ class SQLParams(object):
 			elif isinstance(out_obj, _styles.OrdinalStyle):
 				converter_class = _converting.NamedToOrdinalConverter
 			else:
-				raise TypeError("out_style:{!r} maps to an unexpected type: {!r}".format(
-					out_style,
-					out_obj,
-				))
+				raise TypeError(f"{out_style=!r} maps to an unexpected type: {out_obj!r}")
 
 		elif isinstance(in_obj, _styles.NumericStyle):
 			if isinstance(out_obj, _styles.NamedStyle):
@@ -346,10 +335,7 @@ class SQLParams(object):
 			elif isinstance(out_obj, _styles.OrdinalStyle):
 				converter_class = _converting.NumericToOrdinalConverter
 			else:
-				raise TypeError("out_style:{!r} maps to an unexpected type: {!r}".format(
-					out_style,
-					out_obj,
-				))
+				raise TypeError(f"{out_style=!r} maps to an unexpected type: {out_obj!r}")
 
 		elif isinstance(in_obj, _styles.OrdinalStyle):
 			if isinstance(out_obj, _styles.NamedStyle):
@@ -359,16 +345,10 @@ class SQLParams(object):
 			elif isinstance(out_obj, _styles.OrdinalStyle):
 				converter_class = _converting.OrdinalToOrdinalConverter
 			else:
-				raise TypeError("out_style:{!r} maps to an unexpected type: {!r}".format(
-					out_style,
-					out_obj,
-				))
+				raise TypeError(f"{out_style=!r} maps to an unexpected type: {out_obj!r}")
 
 		else:
-			raise TypeError("in_style:{!r} maps to an unexpected type: {!r}".format(
-				in_style,
-				in_obj,
-			))
+			raise TypeError(f"{in_style=!r} maps to an unexpected type: {in_obj!r}")
 
 		# Create converter.
 		converter = converter_class(
@@ -389,13 +369,12 @@ class SQLParams(object):
 		"""
 		Create the in-style parameter regular expression.
 
-		*escape_char* (:class:`str` or :data:`None`) is the escape character
-		sed to prevent matching an in-style parameter.
+		*escape_char* (:class:`str` or ``None``) is the escape character to prevent
+		matching an in-style parameter.
 
 		*in_obj* (:class:`._styles.Style`) is the in-style parameter object.
 
-		*out_obj* (:class:`._styles.Style`) is the out-style parameter
-		object.
+		*out_obj* (:class:`._styles.Style`) is the out-style parameter object.
 
 		Returns the in-style parameter regular expression (:class:`re.Pattern`).
 		"""
@@ -420,8 +399,8 @@ class SQLParams(object):
 		"""
 		Create the regular expressions to strip comments.
 
-		*strip_comments* (:class:`Sequence` or :data:`None`) contains the
-		comment styles to remove.
+		*strip_comments* (:class:`~collections.abc.Sequence` or ``None``) contains
+		the comment styles to remove.
 
 		Returns the regular expressions (:class:`list` of :class:`re.Pattern`).
 		"""
@@ -446,53 +425,50 @@ class SQLParams(object):
 				pass
 
 			else:
-				raise TypeError("strip_comments[{}]:{!r} must be either a str or tuple.".format(
-					i,
-					comment_style,
-				))
+				raise TypeError(f"strip_comments[{i}]={comment_style!r} must be either a str or tuple.")
 
 		return out_regexes
 
 	@property
 	def escape_char(self) -> Optional[str]:
 		"""
-		*escape_char* (:class:`str` or :data:`None`) is the escape character
-		used to prevent matching an in-style parameter.
+		*escape_char* (:class:`str` or ``None``) is the escape character used to
+		prevent matching an in-style parameter.
 		"""
 		return self.__escape_char
 
 	@property
 	def expand_tuples(self) -> bool:
 		"""
-		*expand_tuples* (:class:`bool`) is whether to convert tuples into a
-		sequence of parameters.
+		*expand_tuples* (:class:`bool`) is whether to convert tuples into a sequence
+		of parameters.
 		"""
 		return self.__expand_tuples
 
 	def format(
 		self,
-		sql: AnyStr,
+		sql: TSqlStr,
 		params: Union[Dict[Union[str, int], Any], Sequence[Any]],
-	) -> Tuple[AnyStr, Union[Dict[Union[str, int], Any], Sequence[Any]]]:
+	) -> Tuple[TSqlStr, Union[Dict[str, Any], Sequence[Any]]]:
 		"""
-		Convert the SQL query to use the out-style parameters instead of
-		the in-style parameters.
+		Convert the SQL query to use the out-style parameters instead of the
+		in-style parameters.
 
-		*sql* (:class:`str` or :class:`bytes`) is the SQL query.
+		*sql* (:class:`LiteralString`, :class:`str`, or :class:`bytes`) is the SQL
+		query.
 
 		*params* (:class:`~collections.abc.Mapping` or :class:`~collections.abc.Sequence`)
-		contains the set of in-style parameters. It maps each parameter
-		(:class:`str` or :class:`int`) to value. If :attr:`.SQLParams.in_style`
-		is a named parameter style. then *params* must be a :class:`~collections.abc.Mapping`.
-		If :attr:`.SQLParams.in_style` is an ordinal parameter style, then
-		*params* must be a :class:`~collections.abc.Sequence`.
+		contains the set of in-style parameters. It maps each parameter (:class:`str`
+		or :class:`int`) to value. If :attr:`.SQLParams.in_style` is a named
+		parameter style. then *params* must be a :class:`~collections.abc.Mapping`.
+		If :attr:`.SQLParams.in_style` is an ordinal parameter style, then *params*
+		must be a :class:`~collections.abc.Sequence`.
 
 		Returns a :class:`tuple` containing:
 
-		-	The formatted SQL query (:class:`str` or :class:`bytes`).
+		-	The formatted SQL query (:class:`LiteralString`, :class:`str` or :class:`bytes`).
 
-		-	The set of converted out-style parameters (:class:`dict` or
-			:class:`list`).
+		-	The set of converted out-style parameters (:class:`dict` or :class:`list`).
 		"""
 		# Normalize query encoding to simplify processing.
 		if isinstance(sql, str):
@@ -502,7 +478,7 @@ class SQLParams(object):
 			use_sql = sql.decode(_BYTES_ENCODING)
 			string_type = bytes
 		else:
-			raise TypeError("sql:{!r} is not a unicode or byte string.".format(sql))
+			raise TypeError(f"{sql=!r} is not a unicode or byte string.")
 
 		# Strip comments.
 		use_sql = self.__strip_comments_from_sql(use_sql)
@@ -521,28 +497,30 @@ class SQLParams(object):
 
 	def formatmany(
 		self,
-		sql: AnyStr,
+		sql: TSqlStr,
 		many_params: Union[Iterable[Dict[Union[str, int], Any]], Iterable[Sequence[Any]]],
-	) -> Tuple[AnyStr, Union[List[Dict[Union[str, int], Any]], List[Sequence[Any]]]]:
+	) -> Tuple[TSqlStr, Union[List[Dict[str, Any]], List[Sequence[Any]]]]:
 		"""
 		Convert the SQL query to use the out-style parameters instead of the
 		in-style parameters.
 
-		*sql* (:class:`str` or :class:`bytes`) is the SQL query.
+		*sql* (:class:`LiteralString`, :class:`str` or :class:`bytes`) is the SQL
+		query.
 
-		*many_params* (:class:`~collections.abc.Iterable`) contains each set
-		of in-style parameters (*params*).
+		*many_params* (:class:`~collections.abc.Iterable`) contains each set of
+		in-style parameters (*params*).
 
 		-	*params* (:class:`~collections.abc.Mapping` or :class:`~collections.abc.Sequence`)
-			contains the set of in-style parameters. It maps each parameter
-			(:class:`str` or :class:`int`) to value. If :attr:`.SQLParams.in_style`
-			is a named parameter style. then *params* must be a :class:`~collections.abc.Mapping`.
-			If :attr:`.SQLParams.in_style` is an ordinal parameter style. then
-			*params* must be a :class:`~collections.abc.Sequence`.
+			contains the set of in-style parameters. It maps each parameter (:class:`str`
+			or :class:`int`) to value. If :attr:`.SQLParams.in_style` is a named
+			parameter style. then *params* must be a :class:`~collections.abc.Mapping`.
+			If :attr:`.SQLParams.in_style` is an ordinal parameter style. then *params*
+			must be a :class:`~collections.abc.Sequence`.
 
 		Returns a :class:`tuple` containing:
 
-		-	The formatted SQL query (:class:`str` or :class:`bytes`).
+		-	The formatted SQL query (:class:`LiteralString`, :class:`str` or
+			:class:`bytes`).
 
 		-	A :class:`list` containing each set of converted out-style
 			parameters (:class:`dict` or :class:`list`).
@@ -555,10 +533,10 @@ class SQLParams(object):
 			use_sql = sql.decode(_BYTES_ENCODING)
 			string_type = bytes
 		else:
-			raise TypeError("sql:{!r} is not a unicode or byte string.".format(sql))
+			raise TypeError(f"{sql=!r} is not a unicode or byte string.")
 
 		if not _util.is_iterable(many_params):
-			raise TypeError("many_params:{!r} is not iterable.".format(many_params))
+			raise TypeError(f"{many_params=!r} is not iterable.")
 
 		# Strip comments.
 		use_sql = self.__strip_comments_from_sql(use_sql)
@@ -578,24 +556,24 @@ class SQLParams(object):
 	@property
 	def in_style(self) -> str:
 		"""
-		*in_style* (:class:`str`) is the parameter style to expect in an SQL
-		query when being parsed.
+		*in_style* (:class:`str`) is the parameter style to expect in an SQL query
+		when being parsed.
 		"""
 		return self.__in_style
 
 	@property
 	def out_style(self) -> str:
 		"""
-		*out_style* (:class:`str`) is the parameter style that the SQL query
-		will be converted to.
+		*out_style* (:class:`str`) is the parameter style that the SQL query will be
+		converted to.
 		"""
 		return self.__out_style
 
 	@property
 	def strip_comments(self) -> Optional[Sequence[Union[str, Tuple[str, str]]]]:
 		"""
-		*strip_comments* (:class:`Sequence` or :data:`None`) contains the
-		comment styles to remove.
+		*strip_comments* (:class:`~collections.abc.Sequence` or ``None``) contains
+		the comment styles to remove.
 		"""
 		return self.__strip_comments
 
